/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EventSwitchManager.java 4798 2009-08-07 12:30:47Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eventswitch.manager;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.mule.umo.manager.UMOManager;
import org.ow2.jasmine.monitoring.eventswitch.endpoints.MBeanCmdWrapper;
import org.ow2.jasmine.monitoring.mbeancmd.CommandDispatcher;

/**
 * The EventSwitch Manager. Here is managed the mule and all MBeanCmd instances.
 *
 * @author Guillaume Renault
 *
 */
public class EventSwitchManager implements EventSwitchManagerMBean {

    // Init phase

    /**
     * The EventSwitch Manager unique instance.
     */
    private static EventSwitchManager manager = null;

    /**
     * Create new instance.
     */
    private EventSwitchManager() {
        this.mbeanCmdList = new HashMap<String, Thread>();
    }

    /**
     * Get the EventSwitch Manager instance.
     * @return the EventSwitch manager unique instance.
     */
    public static EventSwitchManager getInstance() {
        if (EventSwitchManager.manager == null) {
            EventSwitchManager.manager = new EventSwitchManager();
        }

        return EventSwitchManager.manager;
    }

    /**
     * Delete the unique instance. Assumes that only one EventSwitch is launch
     * on the same JVM.
     */
    public static void deleteInstance() {
        EventSwitchManager.manager = null;
    }


    // MULE manager

    /**
     *
     */
    private UMOManager muleManager = null;

    /**
     * @return the muleManager
     */
    public UMOManager getMuleManager() {
        return this.muleManager;
    }

    /**
     * @param muleManager the muleManager to set
     */
    public void setMuleManager(final UMOManager muleManager) {
        this.muleManager = muleManager;
    }


    // MBeanCmd commands manager

    /**
     * MBeanCmd instances' list.
     */
    private Map<String, Thread> mbeanCmdList = null;

    /**
     * Add a MBeanCmd instance to the list.
     *
     * @param name The string representing the command.
     * @param thread The thread in wich the instance is launched.
     */
    public String addMBeanCmdCommand(final String name, final Thread thread) {
        String n = name + ":" + thread.toString();
        this.mbeanCmdList.put(n, thread);
        return n;
    }

    /**
     * Stop a MBeanCmd instance.
     * @param name the name of the instance. It is the string representing the command.
     */
    public void stopMBeanCmdCommand(final String name) {
        this.mbeanCmdList.get(name).interrupt();
        this.mbeanCmdList.remove(name);
    }

    /**
     * Stops all MBeanCmd instances.
     */
    public void stopAllMBeanCmdCommands(){
        for (String name : this.mbeanCmdList.keySet()) {
            this.mbeanCmdList.get(name).interrupt();
        }
        this.mbeanCmdList.clear();
    }

    /**
     * Gets All of the MBeanCmd instance names
     * @return A Set of String wich contains all the MBeanCmd instance names.
     */
    public String[] MBeanCmdCommandNames() {
        Set<String> s = this.mbeanCmdList.keySet();
        String[] result = new String[s.size()];
        s.toArray(result);
        return result;
    }

    /**
     * Start a new MBeanCmd command.
     * @return The command string.
     */
    public String startMBeanCmdCommand(final String command) {
        final String[] args = MBeanCmdWrapper.parseCommandLine(command);
        String name = command;
        if(args.length>0) {
            // Verify that command contains the -jasmine option and a URI
            boolean jasmineOption = false;
            boolean jasmineItem = false;
            for(String arg:args) {
                if(jasmineOption) {
                    if(arg.length()>0) {
                        jasmineItem = true;
                    }
                    break;
                } else if(arg.equals("-jasmine")) {
                    jasmineOption = true;
                }
            }
            if(!jasmineOption || !jasmineItem) {
                throw new IllegalArgumentException("-jasmine option not specified for the MBeanCmd container");
            }

            // Start the command dispatcher's start method (never returns)
            Thread t = new Thread() {
                @Override
                public void run() {
                    CommandDispatcher.start(args);
                }
            };
            name = this.addMBeanCmdCommand(command, t);
            //this.wrappedInstances.add(t);
            t.start();
        }
        return name;
    }

}
